@@ -2,21 +2,24 @@
  * Copyright (c) 2001-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
@@ -79,6 +82,7 @@ private:
     bool			_arSetIsPaused;
     bool			_arSetIsDegrading;
     bool			_arSetIsSyncing;
+    int				_arSetIsSyncingCount;
     bool			_arSetIsEjectable;
     bool			_arSetIsWritable;
     bool			_arSetIsTerminating;
@@ -102,7 +106,7 @@ private:
     UInt32			_arSliceTerminatesActive;
     thread_call_t		_arSliceCloseThreadCall;
     thread_call_t		_arUpdateHeadersThreadCall;
-    thread_call_t		_arSyncronizeCacheThreadCall;
+    thread_call_t		*_arSyncronizeCacheThreadCall;
     IOCommandGate::Action	_arActionChangeSliceMediaState;
     IOBufferMemoryDescriptor	*_arHeaderBuffer;
     AppleRAIDHeader		*_arHeader;
@@ -126,7 +130,8 @@ private:
     virtual IOReturn openSliceMedias(IOOptionBits options, IOStorageAccess access);
     virtual IOReturn closeSliceMedias(IOOptionBits options);
     virtual IOReturn requestSynchronizeCache(void);
-    virtual IOReturn synchronizeCacheSliceMedias(void);
+    virtual IOReturn synchronizeCacheSlice(IOMedia * slice);
+    virtual void completeSynchronizeCacheSlice(void);
     
 protected:
     AppleRAIDEventSource	*arSetEventSource;
@@ -139,7 +144,7 @@ protected:
     UInt32			arHeaderSequence;
     UInt32			*arLogicalSliceNumbers;
     UInt32			*arSliceMediaStates;
-    bool			*arSliceRequestTermiates;
+    bool			*arSliceRequestTerminates;
     IOReturn			*arSliceMediaErrors;
     IOMedia			**arSliceMedias;
     
